@@ -73,7 +73,7 @@ char *home;		    /* invoking user's home directory */
 char *fmhome;		    /* fetchmail's home directory */
 char *program_name;	    /* the name to prefix error messages with */
 flag configdump;	    /* dump control blocks for configurator */
-const char *fetchmailhost;  /* either `localhost' or the host's FQDN */
+char *fetchmailhost;	    /* either `localhost' or the host's FQDN */
 
 #if NET_SECURITY
 void *request = NULL;
@@ -905,7 +905,7 @@ static int load_params(int argc, char **argv, int optind)
     if ((implicitmode = (optind >= argc)))
     {
 	for (ctl = querylist; ctl; ctl = ctl->next)
-	    ctl->active = TRUE;
+	    ctl->active = !ctl->server.skip;
     }
     else
 	for (; optind < argc; optind++) 
@@ -1121,67 +1121,48 @@ static int load_params(int argc, char **argv, int optind)
 
 		ctl->server.truename = xstrdup(leadname);
 	    }
-#ifdef HAVE_GETHOSTBYNAME
 	    else if (ctl->active && ctl->server.dns && !configdump)
 	    {
-		if (ctl->server.authenticate==A_KERBEROS_V4 ||
-		      ctl->server.authenticate==A_KERBEROS_V5 ||
-		    (ctl->server.dns && MULTIDROP(ctl)))
-		{
-		    struct hostent	*namerec;
-
-		    /* compute the canonical name of the host */
-		    errno = 0;
-		    namerec = gethostbyname(ctl->server.queryname);
-		    if (namerec == (struct hostent *)NULL)
-		    {
-			report(stderr,
-			       _("couldn't find canonical DNS name of %s\n"),
-			       ctl->server.pollname);
-			ctl->server.truename = xstrdup(ctl->server.queryname);
-			ctl->server.trueaddr = NULL;
-		    }
-		    else
-			ctl->server.truename=xstrdup((char *)namerec->h_name);
-		}
-#endif /* HAVE_GETHOSTBYNAME */
-		else if (ctl->active)
-		{
-#ifdef HAVE_GETHOSTBYNAME
-		    struct hostent	*namerec;
-		    
-		    /* 
-		     * Get the host's IP, so we can report it like this:
-		     *
-		     * Received: from hostname [10.0.0.1]
-		     *
-		     * For ultra-efficiency, we should find the IP later, when
-		     * we are actually resolving the hostname for a connection.
-		     * Problem is this would have to be done inside SockOpen
-		     * and there's no way to do that that wouldn't both (a)
-		     * be horribly complicated, and (b) blow a couple of
-		     * layers of modularity all to hell.
-		     */
-		    errno = 0;
-		    namerec = gethostbyname(ctl->server.queryname);
-		    if (namerec == (struct hostent *)NULL)
-		    {
-			report(stderr,
-			       _("couldn't find canonical DNS name of %s\n"),
-			       ctl->server.pollname);
-			exit(PS_DNS);
-		    }
-		    else {
-			ctl->server.truename=xstrdup((char *)namerec->h_name);
-			ctl->server.trueaddr=xmalloc(namerec->h_length);
-			memcpy(ctl->server.trueaddr, 
-			       namerec->h_addr_list[0],
-			       namerec->h_length);
-		    }
+#ifndef HAVE_GETHOSTBYNAME
+		ctl->server.truename = xstrdup(ctl->server.queryname);
+		ctl->server.trueaddr = NULL;
 #else
+		struct hostent	*namerec;
+		    
+		/* 
+		 * Get the host's IP, so we can report it like this:
+		 *
+		 * Received: from hostname [10.0.0.1]
+		 *
+		 * For ultra-efficiency, we should find the IP later, when
+		 * we are actually resolving the hostname for a connection.
+		 * Problem is this would have to be done inside SockOpen
+		 * and there's no way to do that that wouldn't both (a)
+		 * be horribly complicated, and (b) blow a couple of
+		 * layers of modularity all to hell.
+		 */
+		errno = 0;
+		namerec = gethostbyname(ctl->server.queryname);
+		if (namerec == (struct hostent *)NULL)
+		{
+		    report(stderr,
+			   _("couldn't find canonical DNS name of %s\n"),
+			   ctl->server.pollname);
 		    ctl->server.truename = xstrdup(ctl->server.queryname);
-#endif /* HAVE_GETHOSTBYNAME */
+		    ctl->server.trueaddr = NULL;
+		    ctl->active = FALSE;
+		    /* use this initially to flag DNS errors */
+		    ctl->wedged = TRUE;
 		}
+		else {
+		    ctl->server.truename=xstrdup((char *)namerec->h_name);
+		    ctl->server.trueaddr=xmalloc(namerec->h_length);
+		    memcpy(ctl->server.trueaddr, 
+			   namerec->h_addr_list[0],
+			   namerec->h_length);
+		    ctl->wedged = FALSE;
+		}
+#endif /* HAVE_GETHOSTBYNAME */
 	    }
 	    else
 		/*
@@ -1267,6 +1248,24 @@ static int load_params(int argc, char **argv, int optind)
 	    run.postmaster = "postmaster";
     }
 
+    /*
+     * If all connections are wedged due to DNS errors, quit.  This is
+     * important for the common case that you just have one connection.
+     */
+    if (querylist)
+    {
+	st = PS_DNS;
+	for (ctl = querylist; ctl; ctl = ctl->next)
+	    if (!ctl->wedged)
+		st = 0;
+	if (st == PS_DNS)
+	{
+	    (void) fprintf(stderr,
+			   _("all mailserver name lookups failed, exiting\n"));
+	    exit(PS_DNS);
+	}
+    }
+
     return(implicitmode);
 }
 
@@ -1300,12 +1299,10 @@ static void terminate_poll(int sig)
 	for (ctl = querylist; ctl; ctl = ctl->next)
 	    if (ctl->smtp_socket != -1)
 	    {
-                /* 
-		 * Don't send QUIT for ODMR case because we're acting
-                 * as a proxy between the SMTP server and client.
-		 */
-                if (ctl->server.protocol != P_ODMR)
-                    SMTP_quit(ctl->smtp_socket);
+		/* don't send QUIT for ODMR case because we're acting
+		   as a proxy between the SMTP server and client. */
+		if (ctl->server.protocol != P_ODMR)
+		    SMTP_quit(ctl->smtp_socket);
 		SockClose(ctl->smtp_socket);
 		ctl->smtp_socket = -1;
 	    }
@@ -1373,7 +1370,7 @@ static const int autoprobe[] =
 static int query_host(struct query *ctl)
 /* perform fetch transaction with single host */
 {
-    int i, st;
+    int i, st = 0;
 
     /*
      * If we're syslogging the progress messages are automatically timestamped.
@@ -1392,7 +1389,8 @@ static int query_host(struct query *ctl)
 	for (i = 0; i < sizeof(autoprobe)/sizeof(autoprobe[0]); i++)
 	{
 	    ctl->server.protocol = autoprobe[i];
-	    if ((st = query_host(ctl)) == PS_SUCCESS || st == PS_NOMAIL || st == PS_AUTHFAIL || st == PS_LOCKBUSY || st == PS_SMTP || st == PS_MAXFETCH)
+	    st = query_host(ctl);
+	    if (st == PS_SUCCESS || st == PS_NOMAIL || st == PS_AUTHFAIL || st == PS_LOCKBUSY || st == PS_SMTP || st == PS_MAXFETCH)
 		break;
 	}
 	ctl->server.protocol = P_AUTO;
