@@ -101,7 +101,7 @@ const unsigned char *host;	/* server hostname */
 		    last_nws = *from;
 		if (*from == '<')
 		    state = 3;
-		else if (*from == '@')
+		else if (*from == '@' || *from == '!')
 		    has_host_part = TRUE;
 		else if (*from == '"')
 		    state = 2;
@@ -145,11 +145,20 @@ const unsigned char *host;	/* server hostname */
 
 	    case 2:	/* we're in a string */
 		if (*from == '"')
-		    state = 1;
+		{
+		    char	*bp;
+		    int		bscount;
+
+		    bscount = 0;
+		    for (bp = from - 1; *bp == '\\'; bp--)
+			bscount++;
+		    if (!(bscount % 2))
+			state = 1;
+		}
 		break;
 
 	    case 3:	/* we're in a <>-enclosed address */
-		if (*from == '@')
+		if (*from == '@' || *from == '!')
 		    has_host_part = TRUE;
 		else if (*from == '>' && from[-1] != '<')
 		{
@@ -306,6 +315,12 @@ const unsigned char *hdr;	/* header to be parsed, NUL to continue previous hdr *
 		state = INSIDE_BRACKETS;
 		tp = 0;
 	    }
+	    else if (*hp == '"')        /* quoted word, copy verbatim */
+	    {
+	        oldstate = state;
+		state = INSIDE_DQUOTE;
+                address[NEXTTP()] = *hp;
+            }
 	    else if (!isspace(*hp)) 	/* just take it, ignoring whitespace */
 		address[NEXTTP()] = *hp;
 	    break;
