@@ -127,8 +127,8 @@ int smtp_open(struct query *ctl)
 					     ctl->server.plugout)) == -1)
 		continue;
 
-           /* return immediately for ODMR */
-           if (ctl->server.protocol == P_ODMR)
+	    /* return immediately for ODMR */
+	    if (ctl->server.protocol == P_ODMR)
                return(ctl->smtp_socket); /* success */
 
 	    /* are we doing SMTP or LMTP? */
@@ -180,11 +180,6 @@ int smtp_open(struct query *ctl)
 
 /* these are shared by open_sink and stuffline */
 static FILE *sinkfp;
-#ifndef HAVE_SIGACTION
-static RETSIGTYPE (*sigchld)(int);
-#else
-static struct sigaction sa_old;
-#endif /* HAVE_SIGACTION */
 
 int stuffline(struct query *ctl, char *buf)
 /* ship a line to the given control block's output sink (SMTP server or MDA) */
@@ -278,7 +273,7 @@ static int send_bouncemail(struct query *ctl, struct msgblk *msg,
     char boundary[BUFSIZ], *bounce_to;
     int sock;
 
-    /* don't bounce  in reply to undeliverable bounces */
+    /* don't bounce in reply to undeliverable bounces */
     if (!msg->return_path[0] || strcmp(msg->return_path, "<>") == 0)
 	return(FALSE);
 
@@ -286,12 +281,13 @@ static int send_bouncemail(struct query *ctl, struct msgblk *msg,
 
     SMTP_setmode(SMTP_MODE);
 
-    strcat(daemon_name, fetchmailhost);
+    /* can't just use fetchmailhost here, it might be localhost */
+    strcat(daemon_name, host_fqdn());
 
     /* we need only SMTP for this purpose */
     if ((sock = SockOpen("localhost", SMTP_PORT, NULL, NULL)) == -1
     		|| SMTP_ok(sock) != SM_OK 
-		|| SMTP_helo(sock, "localhost") != SM_OK
+		|| SMTP_helo(sock, fetchmailhost) != SM_OK
 		|| SMTP_from(sock, daemon_name, (char *)NULL) != SM_OK
 		|| SMTP_rcpt(sock, bounce_to) != SM_OK
 		|| SMTP_data(sock) != SM_OK)
@@ -347,7 +343,8 @@ static int send_bouncemail(struct query *ctl, struct msgblk *msg,
 		char	*error;
 		/* Minimum RFC1894 compliance + Diagnostic-Code field */
 		SockPrintf(sock, "\r\n");
-		SockPrintf(sock, "Final-Recipient: rfc822; %s\r\n", idp->id);
+		SockPrintf(sock, "Final-Recipient: rfc822; %s@%s\r\n", 
+			   idp->id, fetchmailhost);
 		SockPrintf(sock, "Last-Attempt-Date: %s\r\n", rfc822timestamp());
 		SockPrintf(sock, "Action: failed\r\n");
 
@@ -402,10 +399,17 @@ static int handle_smtp_report(struct query *ctl, struct msgblk *msg)
     xalloca(responses[0], char *, strlen(smtp_response)+1);
     strcpy(responses[0], smtp_response);
 
+#ifdef __UNUSED__
+    /*
+     * Don't do this!  It can really mess you up if, for example, you're
+     * reporting an error with a single RCPT TO address among several;
+     * RSET discards the message body and it doesn't get sent to the
+     * valid recipients.
+     */
     SMTP_rset(ctl->smtp_socket);    /* stay on the safe side */
-
     if (outlevel >= O_DEBUG)
 	report(stdout, _("Saved error is still %d\n"), smtperr);
+#endif /* __UNUSED */
 
     /*
      * Note: send_bouncemail message strings are not made subject
@@ -582,7 +586,6 @@ int open_sink(struct query *ctl, struct msgblk *msg,
 	const char	*ap;
 	char		options[MSGBUFSIZE]; 
 	char		addr[HOSTLEN+USERNAMELEN+1];
-	char		**from_responses;
 	int		total_addresses;
 
 	/*
@@ -619,8 +622,13 @@ int open_sink(struct query *ctl, struct msgblk *msg,
 	 * This is a potential problem if the MTAs further upstream
 	 * didn't pass canonicalized From/Return-Path lines, *and* the
 	 * local SMTP listener insists on them. 
+         *
+         * Handle the case where an upstream MTA is setting a return
+         * path equal to "@".  Ghod knows why anyone does this, but 
+	 * it's been reported to happen in mail from Amazon.com and
+	 * Motorola.
 	 */
-	if (!msg->return_path[0])
+	if (!msg->return_path[0] || (0 == strcmp(msg->return_path, "@")))
 	{
 #ifdef HAVE_SNPRINTF
 	    snprintf(addr, sizeof(addr),
@@ -630,7 +638,7 @@ int open_sink(struct query *ctl, struct msgblk *msg,
 		  "%s@%s", ctl->remotename, ctl->server.truename);
 	    ap = addr;
 	}
-	else if (strchr(msg->return_path, '@'))
+	else if (strchr(msg->return_path,'@') || strchr(msg->return_path,'!'))
 	    ap = msg->return_path;
 	else		/* in case Return-Path existed but was local */
 	{
@@ -644,7 +652,12 @@ int open_sink(struct query *ctl, struct msgblk *msg,
 	}
 
 	if (SMTP_from(ctl->smtp_socket, ap, options) != SM_OK)
-	    return(handle_smtp_report(ctl, msg));
+	{
+	    int err = handle_smtp_report(ctl, msg);
+
+	    SMTP_rset(ctl->smtp_socket);    /* stay on the safe side */
+	    return(err);
+	}
 
 	/*
 	 * Now list the recipient addressees
@@ -652,7 +665,6 @@ int open_sink(struct query *ctl, struct msgblk *msg,
 	total_addresses = 0;
 	for (idp = msg->recipients; idp; idp = idp->next)
 	    total_addresses++;
-	xalloca(from_responses, char **, sizeof(char *) * total_addresses);
 	for (idp = msg->recipients; idp; idp = idp->next)
 	    if (idp->val.status.mark == XMIT_ACCEPT)
 	    {
@@ -680,37 +692,7 @@ int open_sink(struct query *ctl, struct msgblk *msg,
 		{
 		    char	errbuf[POPBUFSIZE];
 
-#ifdef __UNUSED__
-		    /*
-		     * I don't remember how this got in here, but it doesn't
-		     * work.  The obvious symptom is that no bounce message
-		     * is sent for a nonexistent user.  Less obviously
-		     * Forwarding to postmaster also does not work. The body is
-		     * discarded.
-		     *
-		     * If a mail is sent to one valid and one invalid
-		     * user, the mail does not go to the valid user
-		     * also as the body is discarded after calling
-		     * RSET!
-		     */
-		    int res;
-		    if ((res = handle_smtp_report(ctl, msg))==PS_REFUSED)
-			return(PS_REFUSED);
-#endif /* __UNUSED__ */
-
-#ifdef HAVE_SNPRINTF
-		    snprintf(errbuf, sizeof(errbuf), "%s: %s",
-				    idp->id, smtp_response);
-#else
-		    strncpy(errbuf, idp->id, sizeof(errbuf));
-		    strcat(errbuf, ": ");
-		    strcat(errbuf, smtp_response);
-#endif /* HAVE_SNPRINTF */
-
-		    xalloca(from_responses[*bad_addresses], 
-			    char *, 
-			    strlen(errbuf)+1);
-		    strcpy(from_responses[*bad_addresses], errbuf);
+		    handle_smtp_report(ctl, msg);
 
 		    (*bad_addresses)++;
 		    idp->val.status.mark = XMIT_RCPTBAD;
@@ -720,10 +702,7 @@ int open_sink(struct query *ctl, struct msgblk *msg,
 			      ctl->listener, addr);
 		}
 	    }
-	if (*bad_addresses)
-	    send_bouncemail(ctl, msg, XMIT_RCPTBAD,
-                            "Some addresses were rejected by the MDA fetchmail forwards to.\r\n",
-                            *bad_addresses, from_responses);
+
 	/*
 	 * It's tempting to do local notification only if bouncemail was
 	 * insufficient -- that is, to add && total_addresses > *bad_addresses
@@ -755,7 +734,7 @@ int open_sink(struct query *ctl, struct msgblk *msg,
 	    {
 		report(stderr, _("can't even send to %s!\n"), run.postmaster);
 		SMTP_rset(ctl->smtp_socket);	/* required by RFC1870 */
-		return(PS_SMTP);
+		return(PS_REFUSED);
 	    }
 
 	    if (outlevel >= O_VERBOSE)
@@ -767,7 +746,10 @@ int open_sink(struct query *ctl, struct msgblk *msg,
 	 * Some listeners (like zmailer) may return antispam errors here.
 	 */
 	if (SMTP_data(ctl->smtp_socket) != SM_OK)
+	{
+	    SMTP_rset(ctl->smtp_socket);    /* stay on the safe side */
 	    return(handle_smtp_report(ctl, msg));
+	}
     }
 
     /*
@@ -958,7 +940,7 @@ int open_sink(struct query *ctl, struct msgblk *msg,
 	memset (&sa_new, 0, sizeof sa_new);
 	sigemptyset (&sa_new.sa_mask);
 	sa_new.sa_handler = SIG_DFL;
-	sigaction (SIGCHLD, &sa_new, &sa_old);
+	sigaction (SIGCHLD, &sa_new, NULL);
 #endif /* HAVE_SIGACTION */
     }
 
@@ -983,12 +965,7 @@ void release_sink(struct query *ctl)
 	    pclose(sinkfp);
 	    sinkfp = (FILE *)NULL;
 	}
-#ifndef HAVE_SIGACTION
-	signal(SIGCHLD, sigchld);
-#else
-	sigaction (SIGCHLD, &sa_old, NULL);
-#endif /* HAVE_SIGACTION */
-	deal_with_sigchld();
+	deal_with_sigchld(); /* Restore SIGCHLD handling to reap zombies */
     }
 }
 
@@ -1007,12 +984,9 @@ int close_sink(struct query *ctl, struct msgblk *msg, flag forward)
 	}
 	else
 	    rc = 0;
-#ifndef HAVE_SIGACTION
-	signal(SIGCHLD, sigchld);
-#else
-	sigaction (SIGCHLD, &sa_old, NULL);
-#endif /* HAVE_SIGACTION */
-	deal_with_sigchld();
+
+	deal_with_sigchld(); /* Restore SIGCHLD handling to reap zombies */
+
 	if (rc)
 	{
 	    report(stderr, 
@@ -1042,10 +1016,14 @@ int close_sink(struct query *ctl, struct msgblk *msg, flag forward)
 	if (SMTP_eom(ctl->smtp_socket) != SM_OK)
 	{
 	    if (handle_smtp_report(ctl, msg) != PS_REFUSED)
+	    {
+	        SMTP_rset(ctl->smtp_socket);    /* stay on the safe side */
 		return(FALSE);
+	    }
 	    else
 	    {
 		report(stderr, _("SMTP listener refused delivery\n"));
+	        SMTP_rset(ctl->smtp_socket);    /* stay on the safe side */
 		return(TRUE);
 	    }
 	}
