@@ -797,14 +797,14 @@ int readheaders(int sock,
 						line,
 						strlen(ctl->server.envelope)))
 		{				
-		    if (skipcount++ != ctl->server.envskip)
+		    if (skipcount++ < ctl->server.envskip)
 			continue;
 		    env_offs = (line - msgblk.headers);
 		}    
 	    }
 	    else if (!received_for && !strncasecmp("Received:", line, 9))
 	    {
-		if (skipcount++ != ctl->server.envskip)
+		if (skipcount++ < ctl->server.envskip)
 		    continue;
 		received_for = parse_received(ctl, line);
 	    }
@@ -1130,7 +1130,7 @@ int readheaders(int sock,
 	return(PS_IOERR);
     }
     else if ((run.poll_interval == 0 || nodetach) && outlevel >= O_VERBOSE && !isafile(2))
-	fputs("#", stderr);
+	fputs("#", stdout);
 
     /* write error notifications */
     if (no_local_matches || has_nuls || bad_addresses)
